*****************************************;
** SAS Scoring Code for PROC Logistic;
*****************************************;

length I_Response $ 9;
label I_Response = 'Into: Response' ;
length U_Response $ 9;
label U_Response = 'Unnormalized Into: Response' ;
format U_Response $9.0;
label P_ResponseUndecided = 'Predicted: Response=Undecided' ;
label P_ResponseAccept = 'Predicted: Response=Accept' ;
label P_ResponseDecline = 'Predicted: Response=Decline' ;

drop _LMR_BAD;
_LMR_BAD=0;

*** Check interval variables for missing values;
if nmiss(Age) then do;
   _LMR_BAD=1;
   goto _SKIP_000;
end;

*** Generate design variables for Treatment;
drop _0_0 _0_1 _0_2 ;
_0_0= 0;
_0_1= 0;
_0_2= 0;
length _st20 $ 20; drop _st20;
_st20 = left(trim(put (Treatment, $20.0)));
if _st20 = 'Free Item'  then do;
   _0_0 = 1;
end;
else if _st20 = 'Buy One Get One Free'  then do;
   _0_1 = 1;
end;
else if _st20 = 'Discount 10%'  then do;
   _0_2 = 1;
end;
else do;
   _0_0 = .;
   _0_1 = .;
   _0_2 = .;
   _LMR_BAD=1;
   goto _SKIP_000;
end;

*** Compute Linear Predictors;
drop _LP0 _LP1;
_LP0 = 0;
_LP1 = 0;

*** Effect: Treatment;
_LP0 = _LP0 + (1.629293638579200E+00) * _0_0;
_LP1 = _LP1 + (3.913577060846000E-01) * _0_0;
_LP0 = _LP0 + (-2.186379373361900E-01) * _0_1;
_LP1 = _LP1 + (-4.852882364699500E-01) * _0_1;
*** Effect: Age;
_LP0 = _LP0 + (2.762719010933300E-03) * Age;
_LP1 = _LP1 + (4.732127165393000E-03) * Age;

*** Predicted values;
drop _LPMAX _MAXP _IY _P0 _P1 _P2;
_LPMAX= 0;
_LP0 = -8.110758193003000E-01 + _LP0;
if _LPMAX < _LP0 then _LPMAX = _LP0;
_LP1 = -1.827713425055600E-01 + _LP1;
if _LPMAX < _LP1 then _LPMAX = _LP1;
_LP0 = exp(_LP0 - _LPMAX);
_LP1 = exp(_LP1 - _LPMAX);
_LPMAX = exp(-_LPMAX);
_P2 = 1 / (_LPMAX + _LP0 + _LP1);
_P0 = _LP0 * _P2;
_P1 = _LP1 * _P2;
_P2 = _LPMAX * _P2;
P_ResponseUndecided = _P0;
_MAXP = _P0;
_IY = 1;
P_ResponseAccept = _P1;
if (_P1 >  _MAXP + 1E-8) then do;
   _MAXP = _P1;
   _IY = 2;
end;
P_ResponseDecline = _P2;
if (_P2 >  _MAXP + 1E-8) then do;
   _MAXP = _P2;
   _IY = 3;
end;
select( _IY );
   when (1) do;
      I_Response = 'Undecided' ;
      U_Response = 'Undecided' ;
   end;
   when (2) do;
      I_Response = 'Accept' ;
      U_Response = 'Accept   ' ;
   end;
   when (3) do;
      I_Response = 'Decline' ;
      U_Response = 'Decline  ' ;
   end;
   otherwise do;
      I_Response = '';
      U_Response = '';
   end;
end;
_SKIP_000:
if _LMR_BAD = 1 then do;
I_Response = '';
U_Response = '';
P_ResponseUndecided = .;
P_ResponseAccept = .;
P_ResponseDecline = .;
end;
